;DEBUG con 1


;==============================================================================
;Description: Lynxmotion custom XBee based radio
;Software version: V2.0
;Programmer: Jim Frye (aka RobotDude), Kurt (aka Kurte), Jeroen Janssen (aka Xan) 
;
;KNOWN BUGS: 
;   - Probably Alot ;) 
; ###### WARNING ##### Work in progress use at your (and your robots) own risk!
;==============================================================================

; 
;Hardware setup: ABB2 with ATOM 28 Pro, XBee, 2 joysticks, 2 sliders, HEX keypad, display 
; 
;NEW IN V1.0 
;   - As released 
; 
; New in V2.0
;	- Converted to use XBEE serial communications.

; 
;KNOWN BUGS: 
;   - Probably Alot ;) 
; 
; [Defines]
TRUE	con		1
FALSE	con		0


;==============================================================================
; [IO Definitions] Lets define what each IO line is used for.
;==============================================================================
;	p0, left vertical joystick 
;	p1, left horizontal joystick
;	p2, right vertical joystick
;	p3, right horizontal joystick
;	in4 					- Col1 on keypad
;	in5 					- Col2 "
;	in6 					- Col3 "
;	in7 					- Col4 "
Display 	con 8 				; LCD Display
Speaker 	con 9 				; Speaker
Row0   	 	con 10 				; Row 0 on keypad
Row1   	 	con 11 				; row 1 on keypad
Row2    	con 12 				; Row 2
Row3    	con 13    			; Row 3
; P14 RXD						; HSERIAL
; P15 TXD						; hserial
RowB		con	17				; Cmd buttons new row
;	p18						- Left slider 
;	p19 					- Right slider 

;==============================================================================
; [EEPROM] Define EEPROM memory locations
;==============================================================================
JoystickRangesDMStart	con	0x0		; 12 bytes - Stores Mins/Maxs of joysticks/sliders
XBeeDMStart				con	0x80	; 4 Bytes - Stores the XBee My and DL we want




;==============================================================================
; [Display Modes]
; We are currently using the "D" key on the keypad to cycle through display 
; modes
;==============================================================================
TransMode				var	byte				; Our current mode
fTModeChanged			var	bit					; has our mode changed?
TMODE_NORMAL			con	0					; Normal Mode
TMODE_DATA				con	1					; Display Data from Robot
TMODE_SELECTDL			con	2					; Select which robot Host to control
TMODE_SELECTMY			con 3					; Change My address
TMODE_CALIBRATE			con	4					; try to calibrate the joysticks...
TMODE_MAX				con	4					; What is our max mode
TransMode = TMODE_NORMAL						; Initialize to normal display mode
fTmodeChanged = FALSE							; the mode has not changed.
bTDataLast				var	byte				; which is the last data we displayed

; Strings we use to display on the LCD
_KEYSTRING				bytetable "Key:   "						
_LJOYSTRING				bytetable "Left:  "
_RJOYSTRING				bytetable "Right: "
_SLIDESTRING			bytetable "Slide: "
_BLANKS					bytetable "                "	; setup for one whole line worth
_LCDLINE2				bytetable 254, 71, 1, 2	; Position to first character in line 2


;==============================================================================
; [XBee and HSERIAL definitions]
; We are also setting up the remote to have several different display and/or 
;==============================================================================

; We are rolling our own communication protocol between multiple XBees, one in
; the receiver (this one) and one in each robot.  We may also setup a PC based
; program that we can use to monitor things...
; Packet format:
; 		Packet Header: <Packet Type><Checksum><SerialNumber><cbExtra>
;
; Packet Types:
XBEE_TRANS_READY		con	0x01	; Transmitter is ready for requests*
XBEE_TRANS_NOTREADY		con 0x02	; Transmitter is exiting transmitting on the sent DL
XBEE_TRANS_DATA			con	0x03	; Data Packet from Transmitter to Robot*
XBEE_TRANS_NEW			con	0x04	; New Data Available

XBEE_RECV_REQ_DATA		con	0x80	;		0x80 - Request Data Packet*
XBEE_RECV_REQ_NEW		con	0x81	;		0x81 - Request Only New data
XBEE_RECV_NEW_THRESH	con 0x82	;		0x82 - Set new Data thresholds
XBEE_RECV_DISP_VAL		con	0x83	; Display a value on line 2

;
; Each of the packets will be defined throughout this file...
;
bPacket					var	byte(8)		; This is the packet of data we will send to the robot.
bPacketPrev				var	byte(8)		; This is the previous data
fPacketChanged 			var byte		; Did the packet change.	
bTemp					var	byte(20)	; Will assume max packet of data for now
cIdleCycles				var	word		; How many cycles through the code have we gone with out a request?
CIDLECYCLESMAX 			con 100			; If I go this many times without sending stuff, resync...
fSendNewPacketMsg		var	bit			; Do we need to send a New packet msg
fNewPacketMsgMode		var	bit			; Are we in New Packet Message Mode?
fNewPacketMsgSent		var	bit			; Have we already sent a new Packet Msg?
; simple defines for HSERSTAT
ENABLEHSERIAL
HSERSTAT_CLEAR_INPUT	con 0 			;Clear input buffer
HSERSTAT_CLEAR_OUTPUT	con	1 			;Clear output buffer
HSERSTAT_CLEAR_BOTH		con	2 			;Clear both buffers
HSERSTAT_INPUT_DATA		con	3 			;If input data is available go to label
HSERSTAT_INPUT_EMPTY	con	4 			;If input data is not available go to label
HSERSTAT_OUTPUT_DATA	con	5 			;If output data is being sent go to label
HSERSTAT_OUTPUT_EMPTY	con	6 			;If output data is not being sent go to label

; Other defines for Xbee
MYADDR					con	$0
DESTADDR				con	$1			; Default DL - can now set by keypad

; XBee data we store in EEPROM
cbXbee					var	byte		; how many bytes are stored in EEPROM
bXBeeCS					var	byte		; checksum
XBeeEE					var	word(2)		; This is the actual data we read from EEPROM
XBeeMY					var	XBeeEE(0)	; First word is our MY
XBeeDL					var	XBeeEE(1)	; Second word is the DL we want to send 2

;==============================================================================
; [Command Buttons]
;==============================================================================

CmdBtns					var	byte							; Cmd Buttons state
; Define which bits are used for which button - May change later as I have the buttons in from R-L where I may want L-R
CMD_UP_MASK				con 0x08							; Logical Up Key
CMD_DOWN_MASK			con	0x04							; Logical Down key
CMD_ENTER_MASK			con	0x02							; Logical Enter key
CMD_ESC_MASK			con	0x01							; logical Esc/Cancel key

CmdBtnsPrev				var	byte							; previous time through


;==============================================================================
; [Timer Tick]
;==============================================================================

; Use wtimer for a timer tick to help us know when to time things out
lTimerTick				var	long		; used with our TimerW clock function
LTIMERTICKTIMEOUT		con 4000000		; I think this is about 2 seconds


;==============================================================================
; [Sliders and Joystick defines ane variables]
;==============================================================================
; Lets define constants for the different ranges of the sliders.
; 
;		Left			Right			Sliders
;		638				630				1023   1023
;  361      598      392   636			  |      |
;       426				416			      0      0

Calibrated 				var bit 		; Are we calibrated
keypress 				var byte		; Character to display on LCD for keypress

col1 					var bit 		; Column values on keypad used in check keypad
col2 					var bit 
col3 					var bit 
col4 					var bit 

; create variables for averaging code 
index 					var byte 
buffer0 				var sword(8) 	; Last 8 values from joystick to generate averages from
buffer1 				var sword(8) 
buffer2 				var sword(8) 
buffer3 				var sword(8) 
buffer4 				var sword(8) 
buffer5 				var sword(8) 

; Our AtoD Sums are stored in the order: RH RV LH LV RS LS
AtoDOrder				bytetable	3, 2, 1, 0, 19, 18
; The next 12 words will be stored in EEPROM, so we can properly generate the values.
AtoDMins				var word(6)
AtoDRanges				var	word(6)

;AToDMins	swordtable 392, 416, 361, 426, 0, 0
;AToDRanges	swordtable 636-392+1, 630-416+1, 598-361+1, 638-426+1, 1023-0+1, 1023-0+1
sums					var	sword(6)	; The sum of the last 8 reads of each joystick/slider for averaging
SumOffsets  			var sword(6) 	; Generated offsets to center joysticks


VSAtoD					var	word


;==============================================================================
; [General defines and variables]
;==============================================================================
i						var	sbyte

; tables of Xs and Ys for the different LCD updates
abLCDUpdX 				bytetable 13, 13, 1, 1, 6, 6		; did not include the ones for the one byte updates, would be 11, 11
abLCDUpdY				bytetable  2,  1, 2, 1, 2, 1		; " 1, 1


; Text tables

;==============================================================================
; [Init] - First make sure all of the IO pins are in the state that they should
; be, and make sure interrupts are enabled
;==============================================================================
;
input p4 
input p5 
input p6 
input p7 
input p17		; our cmd button.
output p10 
output p11 
output p12 
output p13 

;==============================================================================
; Initialize our Timer Tick
lTimerTick = 0
#ifdef TCNT
ONASMINTERRUPT	TIMERWINT, HANDLE_TIMERW_ASM

TCRW = 0x30		; Set the counter to clock/8 which is the slowest we can set it
TMRW = 0x80		; Start timer
enable TIMERWINT_OVF
#endif
;==============================================================================
; Part of the TASerout processing
ONASMINTERRUPT TIMERVINT, TASOIHANDLER
enable


;==============================================================================
; initialize each buffer element, each sum, and then index to 0 
gosub GetSavedJoystickRanges			; Ok lets get our saved away joystick offsets.

; Simple Inits, don't need anything special just zero out as the calibrate function
; will get actual values to summarize
buffer0 = rep 0\8
buffer1 = rep 0\8
buffer2= rep 0\8
buffer3 = rep 0\8
buffer4 = rep 0\8
buffer5 = rep 0\8
SumOffsets = rep 0\6
sums = rep 0\6
index = 0 
CmdBtns = 0			; Assumume no command button has been pressed

Calibrated = 0 ; Mark for calibration
bTDataLast = 0xff	; none of the above...

; chirpy squeak kinda thing 
sound Speaker, [100\880, 100\988, 100\1046, 100\1175] ;musical notes, A,B,C,D. 

; wake up the Matrix Orbital display module 
serout Display ,i19200, [254, 66, 0] ;Backlight on, no timeout. 
serout Display ,i19200, [254, 64, "The D-I-Y XBee  Robot Radio Set!"] ;startup screen. Do only once... 


;==============================================================================
; Initualize the  XBee.
; We will do the init in multiple stages.  We would like to not have very much
; communication going on until both sides are ready.  we will initialize everything
; and then send out a message saying we are ready and then we will wait for the other
; side to start requesting things.  If we don't get any messages for awhile, we will then
; go back to send out a ping saying we are ready...
;
gosub InitXbee
gosub SetXBeeMy[XBeeMy]				; will set our My value from the saved location...
gosub SetXBeeDest[XBeeDL]			; will set the destination address after the initial init
; try displaying the Destination address
gosub GetXBeeDest

gosub XBeeTransReady

gosub ClearHSerialInputBuffer		; try to make sure it is all clear...



;==============================================================================
; --- top of main loop --- 
;==============================================================================
start: 
	; This is where we read everything and setup our packet of data
	; The data is very similar to a PS2 packet of data, with one byte of data per field
	; the data is arranged in teh following byte order:
	;	0 - Buttons High
	;	1 - Buttons Low
	; 	2 - Right Joystick L/R
	;	3 - Right Joystick U/D
	;	4 - Left Joystick L/R
	;	5 - Left Joystick U/D
	; 	6 - Right Slider
	;	7 - Left Slider
	
	gosub CheckKeypad[TRUE]						; This will fill in the two button values in the packet and return an ascii
												; value to display
	; The checkkeypad function may have changed the TransMode
	
	if TransMode = TMODE_CALIBRATE then	
		gosub CalibrateJoySticks
		goto start								; Will wait for new mode...
	elseif TransMode = TMODE_SELECTDL
		gosub UserSelectDL
		goto start								; Processing TDL selected new display mode so go back to start...
	elseif TransMode = TMODE_SELECTMY
		gosub UserSelectMy
		goto start								; Processing TDL selected new display mode so go back to start...
	else
		; If we did a mode change back to a standard input mode tell the other side we are ready again for inputs...
		if fTModeChanged then
			gosub XBeeTransReady
		endif	
	
		; One of our standard outputs	
		cIdleCycles = cIdleCycles + 1			; How many times have I gone through this loop without sending anything?
	
		; averaging expects that the a/d values are < 4096 
		; for each channel 
		;   read the a/d 
		;   subtract the previous value from 8 samples ago from the sum 
		;   store the new value in the circular buffer 
		;   add the new value to the sum 
		;   divide the sum by 8 to get the average value 
		;	Convert to value of 0-255 to fit a byte and emulate PS2 data.
		
		; Right Horizontal	
		sums(0) = sums(0) - buffer0(index) 		; subtract off old value
		adin 3, buffer0(index)				; 
		sums(0) = sums(0) + buffer0(index) 		; Add on new value
		bPacket(2) = ((((((sums(0) + SumOffsets(0)) min (AToDMins(0)*8)) / 8) - AToDMins(0)) * 256) / AToDRanges(0)) max 255
	
		; Right Vertical (p2)
		sums(1) = sums(1) - buffer1(index) 		; subtract off old value
		adin 2, buffer1(index)				;  
		sums(1) = sums(1) + buffer1(index) 		; Add on new value
		bPacket(3) = ((((((sums(1) + SumOffsets(1)) min (AToDMins(1)*8)) / 8) - AToDMins(1)) * 256) / AToDRanges(1)) max 255	; 
	
		; Left Horizontal	
		sums(2) = sums(2) - buffer2(index) 		; subtract off old value
		adin 1, buffer2(index)				;  
		sums(2) = sums(2) + buffer2(index) 		; Add on new value
		bPacket(4) = ((((((sums(2) + SumOffsets(2)) min (AToDMins(2)*8)) / 8) - AToDMins(2)) * 256) / AToDRanges(2)) max 255	; 
	
		; Left Vertical (p2)
		sums(3) = sums(3) - buffer3(index) 		; subtract off old value
		adin 0, buffer3(index)				;  
		sums(3) = sums(3) + buffer3(index) 		; Add on new value
		bPacket(5) = ((((((sums(3) + SumOffsets(3)) min (AToDMins(3)*8)) / 8) - AToDMins(3)) * 256) / AToDRanges(3)) max 255	;
		; Right Slider	
		sums(4) = sums(4) - buffer4(index) 
		adin 19, buffer4(index) 
		sums(4) = sums(4) + buffer4(index)  
		bPacket(6) = (sums(4) / 8) / 4		; should hopefully get us in the range 0-255
		
		; Left Slider
		sums(5) = sums(5) - buffer5(index) 
		adin 18, buffer5(index) 
		sums(5) = sums(5) + buffer5(index)  
		bPacket(7) = (sums(5) / 8) / 4		; should hopefully get us in the range 0-255
		
		; finally increment the index and limit its range to 0 to 7. 
		index = (index + 1) & 7 
	
		fPacketChanged = 0
		fSendNewPacketMsg = FALSE					;
		for i = 0 to 7
			if bPacket(i) <> bPacketPrev(i) then
				if i < 2 then
					fSendNewPacketMsg = TRUE
				elseif (ABS(bPacket(i)-bPacketPrev(i)) > 1)	; BUGBUG:: will be able to set the threholds from robot...
					fSendNewPacketMsg = TRUE
 				endif
 				
				bPacketPrev(i)  = bPacket(i)		; save the new value
				fPacketChanged = fPacketChanged | (1 <<i)		; Say that we changed
			endif
		next
		; 
		if Calibrated=1 then 
			; update the display module 
			gosub CheckAndTransmitDataPacket ; check before and after we output to the LCD

			; Now depending on which mode we are displaying we will either simply display the state of the Joysticks...
			if TransMode = TMODE_NORMAL then
				if index > 5 then
					gosub DoLCDDisplay[11, 1, keypress, 0]	;   serout Display, i19200, [254, 71, 11, 1, keypress] 
				else
					gosub DoLCDDisplay[abLCDUpdX(index), abLCDUpdY(index), 0, bPacket(index+2)]
				endif
			else
				; We are in the Combined display with Line one showing The last thing that changed...
				if (fPacketChanged & %00000011) then ; last 2 bits are for keys
					if bTDataLast <> 0 then
						bTDataLast = 0
						gosub ShowLCDString[1, 1, @_KEYSTRING, 7]
						gosub CheckAndTransmitDataPacket ; check after each string we send...
						gosub TASerout[Display, i19200, @_blanks, 9]
						gosub CheckAndTransmitDataPacket ; check after each string we send...
					endif		

					; now display the actual Character for the keypress
					gosub DoLCDDisplay[6, 1, keypress, 0]	

				elseif (fPacketChanged & %00001100)	; R Joystick X, Y
					if bTDataLast <> 1 then
						bTDataLast = 1
						gosub ShowLCDString[1, 1, @_RJOYSTRING, 7]
						gosub CheckAndTransmitDataPacket ; check after each string we send...
						gosub TASerout[Display, i19200, @_blanks, 9]
						gosub CheckAndTransmitDataPacket ; check after each string we send...
					endif
					
					; now display the joystick values
					gosub DoLCDDisplay[9, 1, 0, bPacket(2)]
					gosub DoLCDDisplay[13, 1, 0, bPacket(3)]
							
				elseif (fPacketChanged & %00110000)	; L Joystick X, Y
					if bTDataLast <> 2 then
						bTDataLast = 2
						gosub ShowLCDString[1, 1, @_LJOYSTRING, 7]
						gosub CheckAndTransmitDataPacket ; check after each string we send...
						gosub TASerout[Display, i19200, @_blanks, 9]
						gosub CheckAndTransmitDataPacket ; check after each string we send...
					endif		
					
					; now display the joystick values
					gosub DoLCDDisplay[9, 1, 0, bPacket(4)]
					gosub DoLCDDisplay[13, 1, 0, bPacket(5)]

				elseif (fPacketChanged & %11000000)	; Sliders R, L
					if bTDataLast <> 3 then
						bTDataLast = 3
						gosub ShowLCDString[1, 1, @_SLIDESTRING, 7]
						gosub CheckAndTransmitDataPacket ; check after each string we send...
						gosub TASerout[Display, i19200, @_blanks, 9]
						gosub CheckAndTransmitDataPacket ; check after each string we send...
					endif		
					
					; now display the joystick values
					gosub DoLCDDisplay[9, 1, 0, bPacket(6)]
					gosub DoLCDDisplay[13, 1, 0, bPacket(7)]

				endif
			endif
			gosub CheckAndTransmitDataPacket
		endif 
		
		if Calibrated=0 & index=0 then 
		  gosub Calibrate 
		endif 
	endif
		
		; do it again ad infinitum 
goto start 



;==============================================================================
; [CheckKeypad] function
;
; Input Parameters:
; 		FCheckModeChange: Should we do checks for mode change? Currently always true.
; 
; Variables updated: 
;		keypress - the ascii value associated with that key.
;		Ver 2.0 - Add 4 buttons hang off of keyboard as an extra row...
; 		change Highs to inputs to keep possibility of dead shorts from happening
;		when pressing multiple switches.
; BUGBUG:: Need to clean this up, maybe table drive?
;==============================================================================
fCheckModeChange var	byte		; should we check for a mode change ?
CheckKeypad[fCheckModeChange]:
	low  Row0 
	input Row1 
	input Row2 
	input Row3 
	input RowB	; buttons we hacked on
	
	col1 = in4 
	col2 = in5 
	col3 = in6 
	col4 = in7 

	bPacket(0) = 0		; assume no buttons pressed
	bPacket(1) = 0
    keypress = " "
    CmdBtnsPrev = CmdBtns
    CmdBtns = 0 
	if fCheckModeChange then
		fTModeChanged = FALSE
	endif

	;Read buttons - New version reads multiple keys as well are down.
	; Process Row 0
	if col1 = 0 then 
		bPacket(0).bit1 = 1
		keypress = "1"
	endif 
	if col2 = 0  then
		bPacket(0).bit2 = 1 
		keypress = "2" 
	endif 
	if col3 = 0 then
	    bPacket(0).bit3 = 1
	    keypress = "3" 
	endif 
	if col4 = 0 then
	    bPacket(1).bit2 = 1 
	    keypress = "A" 
	endif 

	; Process Row 1
	input Row0 
	low  Row1 
	col1 = in4 
	col2 = in5 
	col3 = in6 
	col4 = in7 
	if col1 = 0 then 
	    bPacket(0).bit4 = 1 
	    keypress = "4" 
	endif
	if col2 = 0  then
	    bPacket(0).bit5 = 1 
	    keypress = "5" 
	endif 
	if col3 = 0  then
		bPacket(0).bit6 = 1 
		keypress = "6" 
	endif 
	if col4 = 0 then
		bPacket(1).bit3 = 1 
		keypress = "B" 
	endif 

	; Process Row2
	input Row1 
	low  Row2 
	col1 = in4 
	col2 = in5 
	col3 = in6 
	col4 = in7 
	if col1 = 0 then 
	    bPacket(0).bit7 = 1 
	    keypress = "7" 
	endif 
	if col2 = 0 then
	    bPacket(1).bit0 = 1 
	    keypress = "8" 
	endif 
	if col3 = 0 then
	    bPacket(1).bit1 = 1 
	    keypress = "9" 
	endif 
	if col4 = 0 then
	    bPacket(1).bit4 = 1 
	    keypress = "C" 
	endif 

	; Process Row 3
	input Row2 
	low  Row3 
	col1 = in4 
	col2 = in5 
	col3 = in6 
	col4 = in7 
	if col1 = 0 then 
	    bPacket(0).bit0 = 1 
	    keypress = "0" 
	endif 
	if col2 = 0 then
	    bPacket(1).bit7 = 1 
	    keypress = "F" 
	endif 
	if col3 = 0 then
	    bPacket(1).bit6 = 1 
	    keypress = "E" 
	endif 
	if col4 = 0 then
	    bPacket(1).bit5 = 1 
	    keypress = "D" 
	endif 

	; Process Row B - Our 4 added on buttons.
	input Row3
	low	RowB
	col1 = in4 
	col2 = in5 
	col3 = in6 
	col4 = in7 
	if col1 = 0 then 
	    CmdBtns.bit0 = 1 
	    keypress = "W" 
	endif 
	if col2 = 0 then
	    CmdBtns.bit1 = 1 
	    keypress = "X" 
	endif 
	if col3 = 0 then
	    CmdBtns.bit2 = 1 
	    keypress = "Y" 
	endif 
	if col4 = 0 then
	    CmdBtns.bit3 = 1 
	    keypress = "Z" 
	endif 
	
	; Check to see if the state of the command buttons has changed.
	if fCheckModeChange and (CmdBtns <> CmdBtnsPrev) then			;
		if (CmdBtns & CMD_UP_MASK) and ((CmdBtnsPrev & CMD_UP_MASK) = 0) then
			; CmD Up button has been pressed
			if TransMode = TMODE_MAX then
				TransMode = 0
			else
				TransMode = TransMode + 1
			endif
			fTModeChanged = TRUE
		elseif (CmdBtns & CMD_DOWN_MASK) and ((CmdBtnsPrev & CMD_DOWN_MASK) = 0)
			; CmD Up button has been pressed
			if TransMode = 0 then
				TransMode = TMODE_MAX
			else
				TransMode = TransMode - 1
			endif
			fTModeChanged = TRUE
		endif
		
		if fTModeChanged then
			gosub ClearLCDDisplay
			bTDataLast = 0xff		; make sure we will display something when it changes.
		endif
	endif

	return
	


;==============================================================================
;[Calibrate] Calibrates the  middle positions of the sticks to 1500 
;Calibrates left vertical stick to be total down 
;==============================================================================
Calibrate: 
	gosub ClearLCDDisplay
	serout Display, i19200, [254, 0, "Calibrating..."] 
	pause 1000     

	; For Right Joystick LR and Left joystick LR and UD we want the center values to be 128
	; using this example calculation:
	; bPacket(6) = ((((sums(3) min 3136) / 8) - 392) * 256) / 300	; Make sure bottom > 392 so no negative
	; we see we want the sum to be about 4340 which calculates back out between 128 and 129
	;bPacket(3) = (((((sums(0) + SumOffsets(0)) min (AToDMins(0)*8)) / 8) - AToDMins(0)) * 256) / (AtoDMaxs(0) - AToDMins(0) + 1)
	; Will add a little fudge which is 1/2 the way to the next unit
	SumOffsets(0) = (((128 * AToDRanges(0))/256) + AtoDMins(0)) * 8 + AToDRanges(0)/64 - sums(0)
	SumOffsets(1) = (((128 * AToDRanges(1))/256) + AtoDMins(1)) * 8 + AToDRanges(1)/64 - sums(1)
	SumOffsets(2) = (((128 * AToDRanges(2))/256) + AtoDMins(2)) * 8 + AToDRanges(2)/64 - sums(2)
	
	; For Right Joystick U/D assume at bottom - but do some validations just in case...
	if sums(3) < 3350 then
		SumOffsets(3) =  AtoDMins(3) * 8 + AToDRanges(3)/64 - sums(3)
	else
		SumOffsets(3) = 0
	endif

	gosub ClearLCDDisplay
    
   ; Mark as calibrated 
     Calibrated=1 
return

	
	
;==============================================================================
;[GetSavedJoystickRanges] - This reads in the Saved XBee information. - Currently
; 		we are only readining in the My and Dest(DL) but may also read in other data
;		Start block with Count of bytes as well as a checksum to make sure
;		we are not just reading in Junk...
;		We will start at EEPROM location: 0x0	
;==============================================================================
GetSavedJoystickRanges:

	ReadDm	JoystickRangesDMStart, [cbXBee, bXBeeCS]
#ifdef DEBUG
	serout s_out, i9600, ["Get Saved Joystick ", dec cbXBee, " ", dec bXBeeCS, 13]
#endif		

	if cbXBee = 24 then
		ReadDm JoystickRangesDMStart+2, [str AtoDMins\12, str AtoDRanges\12]
		
		for i = 0 to cbXBee/2
			bXBeeCS = bXBeeCS - (AtoDMins(i).lowbyte + AtoDMins(i).highbyte + AtoDRanges(i).lowbyte + AtoDRanges(i).highbyte)
		next
		
		; If I did the right if valid the cs should be zero
		if bXBeeCS = 0 then
			return;
		endif
	endif
	
	; if we fail for some reason just default to our defaults...
	AToDMins = 392, 416, 361, 426, 0, 0
	AToDRanges = 636-392+1, 630-416+1, 598-361+1, 638-426+1, 1023-0+1, 1023-0+1

	return
	
;==============================================================================
;[SaveJoystickRanges] - This Saves XBee information. - Currently
; 		we are only saving My and Dest(DL) 
;		We will start at EEPROM location: 0x0	
;==============================================================================
SaveJoystickRanges:
	bXBeeCS = 0
	for i = 0 to 5
		bXBeeCS = bXBeeCS + AtoDMins(i).lowbyte + AtoDMins(i).highbyte + AtoDRanges(i).lowbyte + AtoDRanges(i).highbyte
	next
	
	WriteDM JoystickRangesDMStart, [24, bXBeeCS, str AtoDMins\12, str AtoDRanges\12]
	
	return
	



;==============================================================================
;[CalibrateJoySticks] - This function asks the user to move all of the joystick
;	and sliders the entire range and then hit a key, which will save away the
; 	mins and maxs for each of the joysticks as well as center point  
;==============================================================================
wAtoDIn		var	word
wAtoDMins	var	word(6)
wAtoDMaxs	var	word(6)
fError		var	bit

CalibrateJoySticks:
	gosub XBeeTransNotReady		; tell client we are not ready

	; bugbug: May need to to use our TASerout ...
	serout Display, i19200, [254, 0, "Cal Joysticks"] 
	serout Display, i19200,	[254, 71, 0, 2, "Move Full Ranges"]

	;First initialize the mins/maxs to current values
	for i = 0 to 5
		adin AtoDOrder(i), wAtoDMins(i)
		wAtoDMaxs(i) = wAtoDMins(i)
	next
	
	
_CJOY_LOOP:

	; We wish to loop through all 6 joystick/sliders and find the minimum and maximum values for their range.
	; 
	for i = 0 to 5
		adin AtoDOrder(i), wAtoDIn
		if wAtoDIn > wAtoDMaxs(i) then
			wAtoDMaxs(i) = wAtoDIn
		endif
		
		if wAtoDIn < wAtoDMins(i) then
			wAtoDMins(i) = wAtoDIn
		endif
	next

	bPacketPrev(0) = bPacket(0)	; need to save old state...
	bPacketPrev(1) = bPacket(1)
	gosub CheckKeypad[TRUE]		; Will try go get something from keypad - tell it to do Transmode changes...
							
	; check to see if we changed modes...
	if fTModeChanged then
		
		; We have a new mode so return...
		bPacketPrev(0) = bPacket(0)	; Make sure the main loop has our updated state of keys!
		bPacketPrev(1) = bPacket(1)
		return
	endif

	; See if the Enter key is pressed
	if (CmdBtns & CMD_ENTER_MASK) and ((CmdBtnsPrev & CMD_ENTER_MASK) = 0) then
		; So a key was pressed... should check for other consistency but for now good enough.
		
		; But first make sure the ranges look OK...
		fError = FALSE
		; First 4 are for joysticks so assume limited range
		; BUGBUG: could put this part in tables as well
		for i = 0 to 3
			if (wAtoDMins(i) > 450) or (wAtoDMaxs(i) < 575) then
				fError = TRUE
			endif
		next
		;last 2 are for sliders and should have full range...
		for i = 4 to 5
			if (wAtoDMins(i) > 50) or (wAtoDMaxs(i) < 975) then
				fError = TRUE
			endif
		next
		
		if not fError then
			; No errors, so lets update our memory ranges and call the save function
			for i = 0 to 5
				AtoDMins(i) = wAtoDMins(i)
				AtoDRanges(i) = wAtoDMaxs(i) - wAtoDMins(i) + 1
			next
			gosub SaveJoystickRanges
			Calibrated=0				; We will need to recalibrate our center point offsets
			index=0 

			
			serout Display, i19200,	[254, 71, 0, 2, "*** Updated ***"]
		else
			serout Display, i19200,	[254, 71, 0, 2, "***  Error  ***"]
		endif
		
	endif
	
	goto _CJOY_LOOP

return


;==============================================================================
; [ClearHSerialInputBuffer] - This simple helper function will clear out the input
;						buffer from the XBEE
;==============================================================================
ClearHSerialInputBuffer:
	hserin 0, _CIB_TO, [bTemp(0)]

_CIB_TO:
	return



;==============================================================================
; [WaitTransmitComplete] - This simple helper function waits until the HSersial
; 							output buffer is empty before continuing...
; BUGBUG: Rolling our own as the hserstat reset the processor
;==============================================================================
WaitTransmitComplete:
	nop						; transisition to assembly language.
_WTC_LOOP:	
	mov.b	@_HSEROUTSTART,r0l
	mov.b	@_HSEROUTEND,r0h
	cmp.b	r0l,r0h
	bne		_WTC_LOOP:8

	;hserstat 5, WaitTransmitComplete				; wait for all output to go out...	
	return



;==============================================================================
; [InitXBee] - This function will attempt to initialize the XBee, it will check
; 			to see the power level of the VS to see if the power is turned on
;			if so it will init the XBee to the proper startup levels
;==============================================================================
InitXBee:
#ifdef DEBUG
	serout s_out, i9600, ["Init XBee", 13]
#endif		
	gosub ClearLCDDisplay
	serout Display, i19200, [254, 0, "Init XBee..."] 

	gosub GetSavedXBeeInfo			; read in the XBee information
	
#ifdef COMPLETE_XBEE_INIT
	sethserial H9600, H8DATABITS,HNOPARITY,H1STOPBITS

	pause 2000							; have to have a guard time to get into Command sequence
	hserout	["+++"]						; try to get it's attention
	gosub WaitTransmitComplete	
	pause 2000							; have to wait a bit again
	hserout [	"ATNI DIY Transmitter",13,|		; Set a description
				"ATMY ",hex MYADDR, 13, |		; set my address to 1
				"ATGT 3", 13, |					; Set guard time to fast so we don't have to wait long to enter config
				"ATCN", 13]						; exit command mode
#endif	
	pause 1000	; need to wait to exit command mode...
	;sethserial H38400, H8DATABITS,HNOPARITY,H1STOPBITS
	sethserial H62500, H8DATABITS,HNOPARITY,H1STOPBITS
	gosub ClearHSerialInputBuffer

	return


;==============================================================================
; [SetXBeeDest] - Set the XBee destination (DL) to the passed in value.
;
;==============================================================================
newDest	var	word
SetXBeeDest[newDest]:

#ifdef DEBUG
	serout s_out, i9600, ["Set XBee DL: ", hex newDest, 13]
#endif		

	pause 20							; have to have a guard time to get into Command sequence
	hserout	["+++"]						; try to get it's attention
	gosub WaitTransmitComplete	
	pause 20							; have to wait a bit again
	hserout ["ATDL ",hex newDest, 13, |	; Set the destination address
			"ATCN", 13]					; exit command mode
	
	pause 20
	gosub ClearHSerialInputBuffer

	return

;==============================================================================
; [SetXBeeMy] - Set the XBee My Value to the passed in value.
; BUGBUG:: Could combine with other functions...
;
;==============================================================================
SetXBeeMy[newDest]:

#ifdef DEBUG
	serout s_out, i9600, ["Set XBee My: ", hex newDest, 13]
#endif		

	pause 20							; have to have a guard time to get into Command sequence
	hserout	["+++"]						; try to get it's attention
	gosub WaitTransmitComplete	
	pause 20							; have to wait a bit again
	hserout ["ATMY ",hex newDest, 13, |	; Set the destination address
			"ATCN", 13]					; exit command mode
	
	pause 20
	gosub ClearHSerialInputBuffer

	return


;==============================================================================
; [GetXbeeDest] - Get the destination address and display on the LCD
; BUGBUG: sample function, need to clean up.
;
;==============================================================================
GetXBeeDest:
	; Now try seeing what information we can retrieve.
	
#ifdef DEBUG
	serout s_out, i9600, ["Get XBee Dest", 13]
#endif		

;	gosub WaitTransmitComplete	
	pause 20							; have to have a guard time to get into Command sequence
	hserout	["+++"]						; try to get it's attention
	gosub WaitTransmitComplete	
	pause 20							; have to wait a bit again
	hserstat 0							; clear out input buffer
	hserout ["ATDL",13]	; 				Ask for my address
	gosub ClearLCDDisplay
	serout Display, i19200, [254, 0, "Get Dest..."] 
	DO
		hserin 20000, GetXbeeDestTo, [str bTemp\10\13]
	while (bTemp(0)="O") and (bTemp(1)="K")
GetXBeeDestTO:
	hserout ["ATCN",13]	; Exit command mode

	gosub ClearLCDDisplay
    serout Display, i19200, [254, 0, "Dest: ", str bTemp\5\13] 
  	pause 20

	return


;==============================================================================
; [CheckAndTransmitDataPacket] function
; 
; This function will output a packet of data over the XBee to the receiving robot.  This function
; will start off simple and simply dump the raw data.  Then I will build in additional smarts.  Things like:
; only transmit data if it has changed.  Maybe allow the remote robot be able to send us information like
; how much slop should we allow in the deadband range for each channel.  Also Maybe we need to generate
; a pulse to allow the other side know that we are still there...
;==============================================================================

;
;
; local variables
CMDPacket		var	byte(4)
bChkSum			var	byte
bHSerinHasData	var	byte

CheckAndTransmitDataPacket:
	; We will first check to see if there is someone out there who is waiting for this data.
	; They will send us a simple prompt that they want data, at which time we will send out our next
	; packet of data.
;	hserstat HSERSTAT_INPUT_EMPTY, _TP_Timeout			; if no input available quickly jump out.
	; Well Hserstat is failing, try rolling our own.
	mov.b	@_HSERINSTART, r0l
	mov.b	@_HSERINEND, r0h
	sub.b	r0h, r0l
	mov.b	r0l, @BHSERINHASDATA
	if	(not bHSerinHasData) then _TP_Timeout


	; Read in the cmd header.  We expect that the cmd packet will look like:
	; CmdPacket(0) = XBEE_RECV_REQ_DATA
	; CmdPacket(1) = Checksum
	; CmdPacket(2) = sequence Number
	; CmdPacket(3) = Count of extra data in this case 0
	hserin 10000, _TP_Timeout, [str CMDPacket\4]			; Read in a packet header.


	
	; Validate the packet. - Currently only support a simple query...
	; Simple request, no extra data, so checksum should equal the packet number...
	if (CmdPacket(0) = XBEE_RECV_REQ_DATA ) and (CmdPacket(1) = CmdPacket(2)) and (CmdPacket(3) = 0) then
		; First lets generate the checksum
		
		bChkSum = CmdPacket(2) + 8	;
		for i = 0 to 7
			bChkSum = bChkSum + bPacket(i)
		next
		
		; Output the packet data
		; Packet(0) = XBEE_TRANS_DATA
		; Packet(1) = Checksum
		; Packet(2) = Sequence Number in here from request
		; Packet(3) = 8	; 8 extra bytes
		; extra data...
		hserout [XBEE_TRANS_DATA, bChkSum, CmdPacket(2), 8, str bPacket\8]		; Ok we dumped the data to the the output
		gosub ResetTimer		; Reset our timeout timer.
		fSendNewPacketMsg = FALSE												; Clear out that we have new data.
		fNewPacketMsgSent = FALSE												; We have not sent a new Msg since the last data
	; Next check for the packet may be a Req New which changes when we transmite data...	
	elseif (CmdPacket(0) = XBEE_RECV_REQ_NEW ) 
#ifdef DEBUG
		serout s_out, i9600, ["XBEE_RECV_REQ_NEW ", hex CmdPacket(0)\2, hex CmdPacket(1)\2,hex CmdPacket(2)\2,hex CmdPacket(3)\2, 13]
#endif		
		if  (CmdPacket(1) = CmdPacket(2)) and (CmdPacket(3) = 0) then
	
			; Ok valid, if Zero for packet number then back to normal mode else we go into mode where we will tell them when new data is available.
			fNewPacketMsgMode = (CmdPacket(1) <> 0)
#ifdef DEBUG
			serout s_out, i9600, ["fNewPacketMsgMode: ", hex fNewPacketMsgMode, 13]
#endif
		endif

	elseif (CmdPacket(0) = XBEE_RECV_DISP_VAL ) 
		; Try to handle both text and a simple byte value passed to us...
		if (CmdPacket(3) > 0) and (CmdPacket(3) <= 16) then
			; They passed us a string, so lets get that and display it. 
			; BUGBUG:: we are reusing bTemp, make sure this is not a conflict.
			hserin 10000, _TP_Timeout, [str bTemp\CmdPacket(3)]			; Read in a packet header.
			
			; Build checksum
			bChkSum = CmdPacket(2) + CmdPacket(3)	;
			for i = 0 to CmdPacket(3)-1
				bChkSum = bChkSum + bTemp(i)
			next
			if bChkSum = CmdPacket(1) then
				Gosub DisplayRemoteString[@bTemp, CmdPacket(3)]
			endif			
		elseif  (CmdPacket(1) = CmdPacket(2)) and (CmdPacket(3) = 0)
			Gosub DisplayRemoteValue[CmdPacket(2)]
		endif
	else
#ifdef DEBUG
		serout s_out, i9600, ["TP IN:", hex CmdPacket(0)\2, hex CmdPacket(1)\2,hex CmdPacket(2)\2,hex CmdPacket(3)\2, 13]
#endif	
		;We got something we were not expecting so error out.	
		gosub ClearHSerialInputBuffer	; try to clear everything else out.
	endif

_TP_Timeout:
	; Do we have new data to tell the client about?
	if fNewPacketMsgMode and fSendNewPacketMsg and (fNewPacketMsgSent = 0) then
#ifdef DEBUG
		serout s_out, i9600, ["Send New Packet MSG: ", hex fPacketChanged, 13]
#endif		
	
		; We are in new msg mode and the data has changed and we have not told the client about it yet
		hserout [XBEE_TRANS_NEW, 0, 0, 0]		; no extra data no seq number so chksum = 0 also
		fNewPacketMsgSent = TRUE
	endif

	
	; we did not get anything or we did not get a complet packet...
	gosub GetTimerTick		; get the current timer tick
	
	if lTimerTick > LTIMERTICKTIMEOUT then
		gosub ClearHSerialInputBuffer			; clear out our buffer.
		
		gosub XBeeTransReady			; send out another ready signal.
	endif
	return



;==============================================================================
; [XBeeTransReady] - Simple message that we send out when we are ready
;		or if we have not received anything for a long time...
;
;		This packet currently our MY that the destination will use to
;		talk back to us, such that we can have multiple transmitters.
;==============================================================================

XBeeTransReady:
	
	; We will now transmit our My address so the robot knows who the controller is.
	; CmdPacket(0) = XBEE_TRANS_READY
	; CmdPacket(1) = Checksum
	; CmdPacket(2) = Sequence Num = 0
	; CmdPacket(3) = Count of extra data in this case 2
	; 2 bytes of MY
 
	hserout [XBEE_TRANS_READY, 2+XBeeMy.lowbyte+XbeeMy.highbyte, 0, 2, str XBeeMY\2]		; Real simple message 
	cIdleCycles = 0 								; Ok reset our count
	gosub ResetTimer		; Reset our timeout timer.
	return


;==============================================================================
; [XBeeTransNotReady] - Simple message that we send out when are no longer ready
;		to receive anything.  Such as we are in some configuration menu...
;		especially if we may change our MY setting...
;
;		This packet currently has no extra data.
;==============================================================================

XBeeTransNotReady:
	
	; We will now transmit our My address so the robot knows who the controller is.
	; CmdPacket(0) = XBEE_TRANS_READY
	; CmdPacket(1) = Checksum = 0
	; CmdPacket(2) = Sequence Num = 0
	; CmdPacket(3) = Count of extra data 0
	; 2 bytes of MY
 
	hserout [XBEE_TRANS_NOTREADY, 0, 0, 0]		; Real simple message 
	return
	
	
;==============================================================================
;[GetSavedXBeeInfo] - This reads in the Saved XBee information. - Currently
; 		we are only readining in the My and Dest(DL) but may also read in other data
;		Start block with Count of bytes as well as a checksum to make sure
;		we are not just reading in Junk...
;		We will start at EEPROM location: 0x80	
;==============================================================================
GetSavedXBeeInfo:

	ReadDm	XBeeDMStart, [cbXBee, bXBeeCS]
#ifdef DEBUG
	serout s_out, i9600, [""Get Saved XBee Info cb", dec cbXBee, " ", dec bXBeeCS, 13]
#endif		

	if cbXBee = 4 then
		ReadDm XBeeDMStart+2, [str XbeeEE\cbXBee]
#ifdef DEBUG
		serout s_out, i9600, ["DM My", hex xBeeMy, " DL", hex XBeeDL, 13]
#endif		
		
		for i = 0 to cbXBee/2
			bXBeeCS = bXBeeCS - (XBeeEE(i).lowbyte + XBeeEE(i).highbyte)
		next
		
		; If I did the right if valid the cs should be zero
		if bXBeeCS = 0 then
			return;
		endif
	endif
	
	; if we fail for some reason just default to our defaults...
	XBeeMy = MYADDR
	XBeeDL = DESTADDR

	return
	

;==============================================================================
;[SaveXBeeInfo] - This Saves XBee information. - Currently
; 		we are only saving My and Dest(DL) 
;		We will start at EEPROM location: 0x80	
;==============================================================================
SaveXBeeInfo:
	bXBeeCS = 0
	for i = 0 to 1
		bXBeeCS = bXBeeCS + (XBeeEE(i).lowbyte + XBeeEE(i).highbyte)
	next
	
	WriteDM XBeeDMStart, [4, bXBeeCS, str XBeeEE\4]
	
	return
	


;==============================================================================
;[UserSelectDL] - This function shows the current XBee DL in the display and
; 	then allows the user to enter new 16 bit hex address.
;	The Logical Enter key will use to save the new value and Esc/Cancel will
;	reset.
;==============================================================================

;local variables
wNewVal		var	word			; 
bMask		var	byte			; not really needed but
fChanged	var	bit

UserSelectDL:
	gosub XBeeTransNotReady		; tell client we are not ready

_STDL_AGAIN:
; bugbug: May need to to use our TASerout ...
	serout Display, i19200, [254, 71, 0, 1, "Current DL: ", hex4 XBeeDL\4] 
	serout Display, i19200,	[254, 71, 0, 2, "New:"]
	wNewVal = 0			; Start off at zero
	fChanged = FALSE	;

_STDL_LOOP:
	bPacketPrev(0) = bPacket(0)	; need to save old state...
	bPacketPrev(1) = bPacket(1)
	
	gosub CheckKeypad[TRUE]		; Will try go get something from keypad may change mode
		
	; See if we changed modes
	if fTModeChanged then
		bPacketPrev(0) = bPacket(0)	; Make sure the main loop has our updated state of keys!
		bPacketPrev(1) = bPacket(1)
		return
	endif
	
	; Check to see if the Enter or Cancel have been pressed.
	if (CmdBtns & CMD_ENTER_MASK) and ((CmdBtnsPrev & CMD_ENTER_MASK) = 0) then
		if fChanged then 
			; only save if something actually was entered.
			XBeeDL = wNewVal
			gosub SaveXBeeInfo			; Save it away in EEPROM
			gosub SetXBeeDest[XBeeDL]	; Actually set the destination in the XBee
			goto _STDL_AGAIN				; go back and display the new data.
		endif
	elseif (CmdBtns & CMD_ESC_MASK) and ((CmdBtnsPrev & CMD_ESC_MASK) = 0)
		fChanged = FALSE
		wNewVal = 0			; reset
		serout Display, i19200,	[254, 71, 5, 2, hex4 wNewVal\4]
	endif

	; Now check to see if any of the other keys have been pressed. quick test to see if a button pressed
	; and likely that it was not the same as the last pass...
	if ((bPacket(0) <> bPacketPrev(0)) and bPacket(0)) or ((bPacket(1) <> bPacketPrev(1)) and bPacket(1)) then
		for i = 0 to 7
			bMask = 1 << i
			if (bPacket(0) & bMask) and ((bPacketPrev(0) & bMask) = 0) then
				wNewVal = wNewVal * 16 + i
				fChanged = TRUE
			elseif (bPacket(1) & bMask) and ((bPacketPrev(1) & bMask) = 0)
				wNewVal = wNewVal * 16 + i + 8	; Takes care of 8-F
				fChanged = TRUE
			endif
		next
		
		; Display new number.  May actuall not be new yet, but...
		serout Display, i19200,	[254, 71, 5, 2, hex4 wNewVal\4]
	endif

	; And try again...
	goto _STDL_LOOP


;==============================================================================
;[UserSelectMy] - This function shows the current XBee MYL in the display and
; 	then allows the user to enter new 16 bit hex address.
;	The Logical Enter key will use to save the new value and Esc/Cancel will
;	reset.
;==============================================================================

;local variables
UserSelectMy:

	gosub XBeeTransNotReady		; tell client we are not ready

_USMY_AGAIN:
	; bugbug: May need to to use our TASerout ...
	serout Display, i19200, [254, 71, 0, 1, "Current My: ", hex4 XBeeMy\4] 
	serout Display, i19200,	[254, 71, 0, 2, "New:"]
	wNewVal = 0			; Start off at zero
	fChanged = FALSE	;

_USMY_LOOP:
	bPacketPrev(0) = bPacket(0)	; need to save old state...
	bPacketPrev(1) = bPacket(1)
	
	gosub CheckKeypad[TRUE]		; Will try go get something from keypad may change mode
		
	; See if we changed modes
	if fTModeChanged then
		bPacketPrev(0) = bPacket(0)	; Make sure the main loop has our updated state of keys!
		bPacketPrev(1) = bPacket(1)
		return
	endif
	
	; Check to see if the Enter or Cancel have been pressed.
	if (CmdBtns & CMD_ENTER_MASK) and ((CmdBtnsPrev & CMD_ENTER_MASK) = 0) then
		if fChanged then 
			; only save if something actually was entered.
			XBeeMy = wNewVal
			gosub SaveXBeeInfo			; Save it away in EEPROM
			gosub SetXBeeMy[XBeeMY]	; Actually set the My in the XBee
			goto _USMY_AGAIN				; go back and display the new data.
		endif
	elseif (CmdBtns & CMD_ESC_MASK) and ((CmdBtnsPrev & CMD_ESC_MASK) = 0)
		fChanged = FALSE
		wNewVal = 0			; reset
		serout Display, i19200,	[254, 71, 5, 2, hex4 wNewVal\4]
	endif

	; Now check to see if any of the other keys have been pressed. quick test to see if a button pressed
	; and likely that it was not the same as the last pass...
	if ((bPacket(0) <> bPacketPrev(0)) and bPacket(0)) or ((bPacket(1) <> bPacketPrev(1)) and bPacket(1)) then
		for i = 0 to 7
			bMask = 1 << i
			if (bPacket(0) & bMask) and ((bPacketPrev(0) & bMask) = 0) then
				wNewVal = wNewVal * 16 + i
				fChanged = TRUE
			elseif (bPacket(1) & bMask) and ((bPacketPrev(1) & bMask) = 0)
				wNewVal = wNewVal * 16 + i + 8	; Takes care of 8-F
				fChanged = TRUE
			endif
		next
		
		; Display new number.  May actuall not be new yet, but...
		serout Display, i19200,	[254, 71, 5, 2, hex4 wNewVal\4]
	endif

	; And try again...
	goto _USMY_LOOP

;==============================================================================
; [DisplayRemoteString (pStr, cbStr)]
; 
; This function takes care of displaying string and or a number sent to us 
; rom the remote robot.  For now hard coded to a specific location on line 2...
;
; This function will also switch to the appropriate display mode if necessary.
;==============================================================================
pStr	var	pointer
cbStr	var	byte

DisplayRemoteString[pstr, cbStr]
	; Make sure we are in the right mode to display the data
	if TransMode <> TMODE_DATA then
		; BUGBUG: Should put this into a change mode function...
		gosub ClearLCDDisplay
		bTDataLast = 0xff		; make sure we will display something when it changes.
		TransMode = TMODE_DATA
	endif

	gosub TASerout[Display, i19200, @_LCDLINE2, 4]	; hopefully position our self at the start of line 2...

	if cbStr then
		gosub TASerout[Display, i19200, pStr, cbStr]
	endif

	if cbStr < 16 then
		gosub TASerout[Display, i19200, @_blanks, 16-cbStr]
	endif

	return

;==============================================================================
; [DisplayRemoteValue (num)]
; 
; This function takes care of displaying a number sent to us 
; rom the remote robot.  For now hard coded to a specific location on line 2...
;
; This function will also switch to the appropriate display mode if necessary.
;==============================================================================
bVal	var	byte
DisplayRemoteValue[bVal]
	; Make sure we are in the right mode to display the data
	if TransMode <> TMODE_DATA then
		; BUGBUG: Should put this into a change mode function...
		gosub ClearLCDDisplay
		bTDataLast = 0xff		; make sure we will display something when it changes.
		TransMode = TMODE_DATA
	endif

	; now display the value - For now one or the other.
	gosub DoLCDDisplay[13, 2, 0, bVal]

	return
	

;==============================================================================
; [DoLCDDisplay (x, y, char, num)]
; 
; This function takes care of displaying the values to the screen.
; used to build buffers for our own version of serout, may combine later but
; for now leave it seperated as more logic may go in here later...
;==============================================================================
 
bLCDX 		var	byte
bLCDY		var byte
bLCDChar	var	byte
bLCDNum		var	word

abLCDBuff	var	byte(20)			; buffer to use to call TASeroutwith
cbLCDOut	var	word				; how many bytes to output

DoLCDDisplay[bLCDX, bLCDY, bLCDChar, bLCDNum]

	abLCDBuff = 254, 71, bLCDX, bLCDY		; filled in the start of the buffer with the LCD position stuff
	if bLCDChar <> 0 then
		abLCDBuff(4) = bLCDChar
		cbLCDOut = 5
	else
		; May later handle +- types of numbers
		abLCDBuff(4) = dec4 bLCDNum\4
		bLCDChar = 4
		while (bLCDChar < 7) and (abLCDBuff(bLCDChar) = "0")		; reuse the passed in char field as an index
			abLCDBuff(bLCDChar) = " "		; remove leading zeros
			bLCDChar = bLCDChar + 1
		wend
		cbLCDOut = 8
	endif	

	; now lets call our taserout function
	gosub TASerout[Display, i19200, @abLCDBuff, cbLCDOut]
	

	return				

;==============================================================================
; [ClearLCDDisplay]
; 
; This function clears the LCD Display
;==============================================================================
_LCD_CS	bytetable 254,88
ClearLCDDisplay:
	gosub TASerout[Display, i19200, @_LCD_CS, 2]	; Simply call off to our display function
	return
	
;==============================================================================
; [ShowLCDString (x, y, p, cnt)]
; 
; This function takes care of displaying a string on the screen.
;==============================================================================
pb	var	pointer
cb	var	word
ShowLCDString[bLCDX, bLCDY, pb, cb]

	abLCDBuff = 254, 71, bLCDX, bLCDY		; filled in the start of the buffer with the LCD position stuff
	gosub TASerout[Display, i19200, @abLCDBuff, 4]	; first simply output the position we want

	; Then simply output the string buffer
	gosub TASerout[Display, i19200, pb, cb]
	

	return				

;==============================================================================
; [TASerout(pin, baudmode, buffer, bufferlen]
;
; Second attempt will be to use TimerV which is only 8 bits, this version will
; use the Compare Match A interrupt do do the acutual output bit processing.
; The main loop will still wait in for the work to be done.  Could later change
; this to queue the values and do it all in the background, but that could get
; complicated as maybe I would start doing multiple different IO ports at the same
; time...  But maybe later.
;
;==============================================================================

; define parameters to this function.
TASPin var	byte
TASBM	var	word
TASPBUF var	POINTER
TASCnt var	word

; Ok lets define the communication variables that will be used between
; the main function here and the interrupt handler.
TASOIH_R1		var	word			; status bits...
;	rll - The Byte to output
; 	r1h	- Other status
;		bit  7 - Do the start bit
;		bit  6 - Do the Stop bit
;		bits 4-5 - 0 not used
;		bits 0-3 - count of bits left to output...
;       ...
TASOIH_R2		var	word			; Some of the stuff to pass as follows:
;  r2 - the PDRX of the IO pin 
TASOIH_R3		var	word			; More stuff to and from the main function
; r3l - IO pin number on the PDRX for the BAP io pin
; r3h - The byte to output.


TASerout[TASPin, TASBM, TASPBUF, TASCNT]
								; BUGBUG: could do this in assembly as well

	; ok need to initialize TimerV
; transistion to assembly...
	mov.w	@TASBM:16, r1			; get the baudmode.
	bld.b	#6, r1h					; Get the normal/invert bit into C
	subx.b	r3h, r3h				; r3h = 0 if Normal or ff if inverted
	and.w	#0x1fff, r1				; extract the bit time
	mov.w	#3,e1					; start off assuming clock will be divided by 8 as the baud mode is set this way
_TASOMCL:	
	or.b	r1h, r1h				; see if our count is < 256
	beq		_TASOBMLD:8				; yep done with this loop
	shlr.w	r1						; nope divide by 2
	add.w	#1, e1					; change clock input for TimerV
	jmp		_TASOMCL:8				; Check again

_TASOBMLD:	
	; BUGBUG: should verify that we are in a valid range...
	mov.b	r1l, @TCORA:8			; save away the value that we should count up to for each bit
	mov.w	e1, r1
	mov.b	#0, r1h					; zero out the high byte
	rotr.w	r1						; so low order bit now in high bit of high word
	rotl.b	r1h						; now in low bit of r1h
	mov.b	r1h, @TCRV1:8			; OK so should have the low bit of timer calc stored properly away.
	or.b	#0x08, r1l				; Added in the bits: CCLR0 for the TCRV0
	mov.b	r1l, @TCRV0:8			; save away the generated configuration byte
		
; first lets use the Porttable that BAP has to convert logical pin to Port/pin
	xor.l	er0, er0				; zero out er0
	mov.b	@TASPIN:16, r0l			; move the logical pin number into r0l
	and.b	#0x3f, r0l				; make sure we are in the range 0-63
	shll.b	r0l						; double the offset as the port table is 2 bytes per entry.
	mov.w	@(_PORTTABLE:16, er0), r2	; OK R2 now has the two bytes high being the bit, and low being the port...
	mov.b	r2h, r3l				; Ok r3l now has the pin number
	mov.b	#0xff, r2h				; setup 32 bit address to port register PDRx
	
	; make sure it is setup for output
	mov.b	@(0x70-0xD0, er2), r0l	; get the current value for PCRx byte from the shadow location
	bset.b	r3l, r0l				; set the appropriate bit for our port
	mov.b	r0l, @(0x70-0xD0, er2)	; save away the updated value to the shadow location
	mov.b	r0l, @(0x10, er2)		; save away in the actual PCRx byte
		
	; Make sure the IO line is correct to start off with
	or.b	r3h, r3h
	beq		_TASOINORMAL:8			; OK will start up normal
	bset.b	r3l,@er2				; Inverted mode start high
	jmp		_TASOINITBUF:8			; continue to setup pointers to buffers
_TASOINORMAL:
	bclr.b	r3l, @er2				; this sets the IO line to low...

	; Now setup data pointer and counter.
_TASOINITBUF:	
	mov.l	@TASPBUF:16, er4			; OK now ER4 has pointer to the buffer to output.
	mov.w	@TASCNT:16, e3			; Now E3 has the count of bytes to output

	; put up to one bit delay at the start to get everything in sync...
_TASOWAB:	
	bld.b	#6, @TCSRV:8			; see if we had a timer cycle go through
	bcc		#_TASOWAB:8				; nope
	bclr.b	#6, @TCSRV:8			; clear out the status

	; save away some of the data that does not change for the interrupt.
	mov.w	r2, @TASOIH_R2:16		; save away our the PDrx value
	mov.w	r3, @TASOIH_R3:16		; save awty the port pin number and inversion state.
	
	
;------------------------------------------------
; Main output loop: helps to know what each register is doing:
; r2	- Word address for PDRx for the output pin
; r3l	- bit number of IO PIN
; r3h	- 0 if Normal communication, FF if inverted.
; e3	- Count of bytes left to output	
; er4 	- Pointing to the current byte to output
	
_TASOBYTELOOP:
	or.w	e3, e3					; see if more bytes left to go
	beq		_TASODONE:16			; NO BYtes left go to do the cleanup
	mov.b	@er4+, r1l				; Ok have the next byte to output
	xor.b	r3h,r1l					; Hack if inverted mode we will invert all of the bits here...
	
	mov.b	#0x88, r1h				; status of do start bit and 8 bits to loop through
	mov.w	r1, @TASOIH_R1:16		; save away byte, plus status byte
	
	; align our self with the bit clock to make sure we don't give to small of a start pulse
_TASOWSB0:	
	bld.b	#6, @TCSRV:8			; see if we had a timer cycle go through
	bcc		#_TASOWSB0:8			; nope
	bclr.b	#6, @TCSRV:8			; clear out the status
	

	; now lets enable the interrupt and wait until the interrupt handler tells us it's done
	bset.b	#6, @TCRV0:8			; OK turned on TCRV0
		
	; now lets wait until the interrupt handler says it is done with that byte.
_TASO_WAITFORBYTEDONE:
	mov.w	@TASOIH_R1, r1			; get the status byte
	bne		_TASO_WAITFORBYTEDONE:8	; not done yet.
				
	; now jump up to see if there are any more bytes left to output
	dec.w	#1,e3						; decrement the count and go back up to check to see if we are done
	bra		_TASOBYTELOOP:8
		

_TASODONE:	
; transistion back to basic	
return



;==============================================================================
; TASOIHandler
;
; This version will use the Compare Match Interrupt A to do all of the manipulation
; of the IO line.  This should keep the timing hopefully very consistent from
; bit to bit.  
;
; will need to use some memory locations to store the state information between
; interrupts as other interupts may be involved, which could cause any register
; using between the main function and the interrupt to be corrupted.
;
;==============================================================================

BEGINASMSUB 
TASOIHANDLER:
	push.w	r3					; save away...
	push.w	r2 					; first save away registers we will touch
	push.w	r1					;
	
	mov.w	@TASOIH_R1:16, r1	; now get all of our state information to use
	mov.w	@TASOIH_R2, r2		;
	mov.w	@TASOIH_R3, r3		;
	
	bclr.b	#6, @TCSRV:8		; clear out the interrupt bit so it can happen again
;	andc	#0x7f,ccr               ; allow other interrupts to happen 

	;	OK lets figure out what we need to do.
	; first check to see if we are doing a start bit
	;
	bld.b	#7, r1h				; the bit value goes into C bit
	bcc		_TASOIH_CHECKFORSTOPBIT:8
	bclr.b	#7, r1h				; clear it out
		
	; Ok lets do the start bit
	; See if we are inverted
	or.b	r3h,r3h				; see if we are in inverted mode
	bne		_TASOIH_STARTINVERT:8
	; normal inverted.
	bset.b	r3l,@er2				; Mormal mode Start goes high
	bra		_TASOIH_CLEANUP:8
_TASOIH_STARTINVERT:
	bclr.b	r3l, @er2				; this sets the IO line to low...
	bra		_TASOIH_CLEANUP:8		
		
_TASOIH_CHECKFORSTOPBIT:
	; Now test for Stop bit
	bld.b	#6, r1h
	bcc		_TASOIH_DODATABIT:8

	
	; Ok lets do the stop bit
	; See if we are inverted
	or.b	r3h, r3h
	bne		_TASOIH_STOPINVERT:8
	; normal inverted.
	bclr.b	r3l, @er2				; this sets the IO line to low...
	bra		_TASOIH_STOPCLEANUP:8

_TASOIH_STOPINVERT:
	bset.b	r3l,@er2				; Mormal mode Start goes high

_TASOIH_STOPCLEANUP:
	xor.w	r1,r1					; clear out all status - will trigger main function

	; Ok this byte is done, turn off the interrupt
	bclr.b	#6, @TCRV0:8			; OK turned off TCRV0
	bra		_TASOIH_CLEANUP:8		
	
_TASOIH_DODATABIT:
	; OK lets do the normal IO...

	shlr.b	r1l						; ok lets shift it down lowest bit goes into carry.
	bcc		_TASOIH_BIT0:8				; not set so was zero
	bclr.b	r3l,@er2				; The bit is a zero
	jmp		_TASOIH_BITCLEANUP:8			; 
_TASOIH_BIT0:
	bset.b	r3l, @er2				; Ok this bit goes high
	
_TASOIH_BITCLEANUP:
	; now see if we finished doing all of the bits
	dec.b	r1h						; decrement the number of bits left to output
	bne		_TASOIH_CLEANUP:8		; still more to go, get ready to return from interrupt

	; we have output all 8 bits, setup to do stop bit
	bset.b	#6, r1h					; set the do the invert bit to process

_TASOIH_CLEANUP:		
	; save away the values that have changed for the next interrupt...
	mov.w	r1,@TASOIH_R1:16
	;mov.w	r2,@TASOIH_R2:16	; does not change...
	;mov.w	r3,@TASOIH_R3:16	; does not change
	
	pop.w	r1					; restore back the registers we used to the caller
	pop.w	r2
	pop.w	r3 

	rte							; and return from the exception.

ENDASMSUB 

;==============================================================================
;[HANDLE_TIMERW_ASM] - Handle timer W overlfow in assembly language. 
;		Currently not as optimized as possible as I could simply load the
;		high word, but this makes it more foolproof.
;==============================================================================

   BEGINASMSUB 
HANDLE_TIMERW_ASM 
	push.l 	er1                 ; first save away ER1 as we will mess with it. 
	bclr 	#7,@TSRW:8          ; clear the cooresponding bit in the interrupt pending mask 
	mov.l 	@LTIMERTICK:16,er1  ;  
	inc.w 	#1,e1 				; Increment the highword of our 32 bit counter
	mov.l	er1, @LTIMERTICK:16 
	pop.l 	er1 
	rte 
	ENDASMSUB 

;==============================================================================
;[getTimerTick] - Gets the Timer value from our overflow counter as well as the TCA counter.  It
;                makes sure of consistancy. That is it is very posible that 
;                after we grabed the timers value it overflows, before we grab the other part
;                so we check to make sure it is correct and if necesary regrab things.
;==============================================================================

wTimerTmp	var	long
GetTimerTick:	
	wTimerTmp = lTimerTick.highword
	lTimerTick.lowword = TCNT
	if lTimerTick.highword <> wTimerTmp then GetTimerTick	; make sure consistent value
	return

;==============================================================================
; [ResetTimer]
;==============================================================================
ResetTimer:
	TCNT = 0
	lTimerTick = 0
	return